﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;
using Infragistics.Win.UltraWinGrid;

namespace _137325
{
    public partial class MyFormControl : Form
    {

        BackgroundWorker bw = new BackgroundWorker();

        private static Random random = new Random((int)DateTime.Now.Ticks);
        
        public MyFormControl()
        {
            
            InitializeComponent();
            ultraGrid1.DataSource = GenerateDataSet();
            bw.WorkerReportsProgress = true;
            bw.WorkerSupportsCancellation = true;

            bw.DoWork += new DoWorkEventHandler(bw_DoWork);
            bw.ProgressChanged += new ProgressChangedEventHandler(bw_ProgressChanged);
            bw.RunWorkerCompleted += new RunWorkerCompletedEventHandler(bw_RunWorkerCompleted);

        }

        void bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
        
            if (e.Result is Exception)
            { 
                Exception ex = e.Result as Exception;
                MessageBox.Show(ex.Message, ex.GetType().Name);
            }


        }

        void bw_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            Debug.WriteLine(string.Format("{0}% compleated", e.ProgressPercentage));
        }

        void bw_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            if (e.Argument is DataRow)
            {
               
                DataRow parenData = (DataRow)e.Argument;
                try
                {

                int totalRecord = 0;
                foreach (DataRelation rel in parenData.Table.ChildRelations)
                {
                    DataRow[] childRows = parenData.GetChildRows(rel);
                    totalRecord += childRows.Length;
                }

                int processecedRecords = 0;
                foreach (DataRelation rel in parenData.Table.ChildRelations)
                {
                    DataRow[] childRows = parenData.GetChildRows(rel);
                    //see whihc table this relation reffers
                    //in order to make proper calculation
                    switch (rel.ChildTable.TableName)
                    {
                        default:
                            foreach (DataRow cRow in childRows)
                            {
                                //simulate time consuming operation 
                                Thread.Sleep(10);
                                cRow["Random Calculated Text"] = RandomString(7);
                                Debug.WriteLine("UpdatewasDone ");
                            }
                            
                            break;
                    }
                    
                    processecedRecords += childRows.Length;
                    worker.ReportProgress((int)(processecedRecords * 100 / totalRecord));
                }

                }
                catch (Exception ex)
                {
                    e.Cancel = true;
                    e.Result = ex;
                }
            }
            else
            { 
                e.Cancel = true;
                e.Result = new ArgumentException("Argument is not valid. Argument must be of type UltraGridRow");
            }
               
        }

        public DataSet GenerateDataSet()
        {
            DataSet RetDataSet = new DataSet();
            DataTable parent = RetDataSet.Tables.Add("parent");

            parent.Columns.Add("id", typeof(int)).AutoIncrement = true;
            parent.Columns.Add("Column 1", typeof(string)).Expression = "'Row' + [id]";

            DataTable child1 = RetDataSet.Tables.Add("child1");

            child1.Columns.Add("pid", typeof(int));
            child1.Columns.Add("Random Calculated Text", typeof(string));
            child1.Columns.Add("id", typeof(int)).AutoIncrement = true;

            DataTable child2 = RetDataSet.Tables.Add("child2");

            child2.Columns.Add("pid", typeof(int));
            child2.Columns.Add("Random Calculated Text", typeof(string));
            child2.Columns.Add("id", typeof(int)).AutoIncrement = true;

            RetDataSet.Relations.Add("parentChild1", parent.Columns["id"], child1.Columns["pid"]);
            RetDataSet.Relations.Add("parentChild2", parent.Columns["id"], child2.Columns["pid"]);

            for (int i = 0; i < 10; i++)
            {
                DataRow parentR = parent.Rows.Add();

                child1.Rows.Add(parentR["id"], RandomString(7));
                child1.Rows.Add(parentR["id"], RandomString(7));

                child2.Rows.Add(parentR["id"], RandomString(7));
                child2.Rows.Add(parentR["id"], RandomString(7));
                child2.Rows.Add(parentR["id"], RandomString(7));
                child2.Rows.Add(parentR["id"], RandomString(7));

            }
            return RetDataSet;

        }

        private void ultraGrid1_ClickCell(object sender, Infragistics.Win.UltraWinGrid.ClickCellEventArgs e)
        {
            if (!bw.IsBusy)
            {
               

                UltraGridRow row = e.Cell.Row;

                //this menat that chile row was cliked
                if (row.HasParent())
                    return;
                DataRow parenData = ((DataRowView)row.ListObject).Row;

                bw.RunWorkerAsync(parenData);

            }

        }

        private string RandomString(int size)
        {
            StringBuilder builder = new StringBuilder();
            char ch;
            for (int i = 0; i < size; i++)
            {
                ch = Convert.ToChar(Convert.ToInt32(Math.Floor(26 * random.NextDouble() + 65)));
                builder.Append(ch);
            }

            return builder.ToString();
        }

        private void ultraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.Override.CellClickAction = CellClickAction.RowSelect;
        }
        
    }

   
}
